// src/Benchmark.tsx
import * as React from "react";

// src/timing.ts
var NS_PER_MS = 1e6;
var MS_PER_S = 1e3;
function now() {
  if (typeof window !== "undefined" && window.performance) {
    return window.performance.now();
  } else if (typeof process !== "undefined" && process.hrtime) {
    const [seconds, nanoseconds] = process.hrtime();
    const secInMS = seconds * MS_PER_S;
    const nSecInMS = nanoseconds / NS_PER_MS;
    return secInMS + nSecInMS;
  } else {
    return Date.now();
  }
}

// src/math.ts
var getStdDev = (values) => {
  const avg = getMean(values);
  const squareDiffs = values.map((value) => {
    const diff = value - avg;
    return diff * diff;
  });
  return Math.sqrt(getMean(squareDiffs));
};
var getMean = (values) => {
  const sum = values.reduce((sum2, value) => sum2 + value, 0);
  return sum / values.length;
};
var getMedian = (values) => {
  if (values.length === 1) {
    return values[0];
  }
  const numbers = values.sort((a, b) => a - b);
  return (numbers[numbers.length - 1 >> 1] + numbers[numbers.length >> 1]) / 2;
};

// src/Benchmark.tsx
var sortNumbers = (a, b) => a - b;
var initialState = {
  running: false,
  startTime: 0,
  cycle: 0,
  samples: []
};
function BenchmarkInner({
  component: Component,
  componentProps,
  includeLayout = false,
  onComplete,
  samples: numSamples,
  timeout = 1e4,
  type = "mount"
}, ref) {
  const [{ running, cycle, samples, startTime }, dispatch] = React.useReducer(reducer, initialState);
  React.useImperativeHandle(ref, () => ({
    start: () => {
      dispatch({ type: "START", payload: now() });
    }
  }));
  const shouldRender = getShouldRender(type, cycle);
  const shouldRecord = getShouldRecord(type, cycle);
  const isDone = getIsDone(type, cycle, numSamples);
  const handleComplete = React.useCallback(
    (startTime2, endTime, samples2) => {
      const runTime = endTime - startTime2;
      const sortedElapsedTimes = samples2.map(({ elapsed }) => elapsed).sort(sortNumbers);
      const mean = getMean(sortedElapsedTimes);
      const stdDev = getStdDev(sortedElapsedTimes);
      const result = {
        startTime: startTime2,
        endTime,
        runTime,
        sampleCount: samples2.length,
        samples: samples2,
        max: sortedElapsedTimes[sortedElapsedTimes.length - 1],
        min: sortedElapsedTimes[0],
        median: getMedian(sortedElapsedTimes),
        mean,
        stdDev,
        p70: mean + stdDev,
        p95: mean + stdDev * 2,
        p99: mean + stdDev * 3,
        layout: void 0
      };
      if (includeLayout) {
        const sortedLayoutTimes = samples2.map(({ layout }) => layout).sort(sortNumbers);
        const mean2 = getMean(sortedLayoutTimes);
        const stdDev2 = getStdDev(sortedLayoutTimes);
        result.layout = {
          max: sortedLayoutTimes[sortedLayoutTimes.length - 1],
          min: sortedLayoutTimes[0],
          median: getMedian(sortedLayoutTimes),
          mean: mean2,
          stdDev: stdDev2,
          p70: mean2 + stdDev2,
          p95: mean2 + stdDev2 * 2,
          p99: mean2 + stdDev2 * 3
        };
      }
      onComplete(result);
      dispatch({ type: "RESET" });
    },
    [includeLayout, onComplete]
  );
  React.useMemo(() => {
    if (running && shouldRecord) {
      dispatch({ type: "START_SAMPLE", payload: now() });
    }
  }, [cycle, running, shouldRecord]);
  React.useEffect(() => {
    if (!running) {
      return;
    }
    const now2 = now();
    if (shouldRecord && samples.length && samples[samples.length - 1].end < 0) {
      if (includeLayout && type !== "unmount" && document.body) {
        document.body.offsetWidth;
      }
      const layoutEnd = now();
      dispatch({ type: "END_SAMPLE", payload: now2 });
      dispatch({ type: "END_LAYOUT", payload: layoutEnd - now2 });
      return;
    }
    const timedOut = now2 - startTime > timeout;
    if (!isDone && !timedOut) {
      setTimeout(() => {
        dispatch({ type: "TICK" });
      }, 1);
      return;
    } else if (isDone || timedOut) {
      handleComplete(startTime, now2, samples);
    }
  }, [includeLayout, running, isDone, samples, shouldRecord, shouldRender, startTime, timeout]);
  return running && shouldRender ? /* @__PURE__ */ React.createElement(Component, {
    ...componentProps,
    "data-testid": cycle
  }) : null;
}
var Benchmark = React.forwardRef(BenchmarkInner);
function reducer(state = initialState, action) {
  switch (action.type) {
    case "START":
      return {
        ...state,
        startTime: action.payload,
        running: true
      };
    case "START_SAMPLE": {
      const samples = [...state.samples];
      samples.push({ start: action.payload, end: -Infinity, elapsed: -Infinity, layout: -Infinity });
      return {
        ...state,
        samples
      };
    }
    case "END_SAMPLE": {
      const samples = [...state.samples];
      const index = samples.length - 1;
      samples[index].end = action.payload;
      samples[index].elapsed = action.payload - samples[index].start;
      return {
        ...state,
        samples
      };
    }
    case "END_LAYOUT": {
      const samples = [...state.samples];
      const index = samples.length - 1;
      samples[index].layout = action.payload;
      return {
        ...state,
        samples
      };
    }
    case "TICK":
      return {
        ...state,
        cycle: state.cycle + 1
      };
    case "RESET":
      return initialState;
    default:
      return state;
  }
}
function getShouldRender(type, cycle) {
  switch (type) {
    case "mount":
    case "unmount":
      return !((cycle + 1) % 2);
    case "update":
      return true;
    default:
      return false;
  }
}
function getShouldRecord(type, cycle) {
  switch (type) {
    case "mount":
      return !((cycle + 1) % 2);
    case "update":
      return cycle !== 0;
    case "unmount":
      return !(cycle % 2);
    default:
      return false;
  }
}
function getIsDone(type, cycle, numSamples) {
  switch (type) {
    case "mount":
    case "unmount":
      return cycle >= numSamples * 2 - 1;
    case "update":
      return cycle >= numSamples;
    default:
      return true;
  }
}
export {
  Benchmark
};
//# sourceMappingURL=index.js.map
